local uci = require "luci.model.uci"
local fs    = require "nixio.fs"
local util	= require "luci.util"
local sys   = require "luci.sys"
local fw = require("luci.model.firewall").init()

local ipairs, string, type, math, tonumber, io = ipairs, string, type, math, tonumber, io

module "luci.util_tlt"

local net_map = {
	{ ifname_reg = "eth1", type = "wired", proto = {"static", "dhcp", "pppoe"}},
	{ ifname_reg = "eth0", type = "wired_lan", proto = {"static", "dhcp"}},
	{ ifname_reg = "wwan", type = "mobile", proto = "wwan"},
	{ ifname_reg = "wlan", type = "wifi", proto = "dhcp"},
	{ ifname_reg = "connm", type = "mobile", proto = "connm"}
}

function show_mobile()
	local cur = uci.cursor()
	local number = cur:get("hwinfo", "hwinfo", "mnf_code")

	if number then
		number = number:sub(7, 7)
		if number == "N" then
			return false
		end
	end

	return true
end

function gps_on()
	local cur = uci.cursor()
	local is_hw = tonumber(cur:get("hwinfo", "hwinfo", "gps"))

	if is_hw == 1 then
		local is_on = tonumber(cur:get("gps", "gps", "enabled"))
		if is_on == 1 then
			return true
		end
	end
	return false
end

function convert_bytes(bytes, unit)
	local map = {
		b = 1,
		kb = 1000,
		mb = 1000 * 1000,
		gb = 1000 * 1000 * 1000
	}

	if type(bytes) ~= "number" then
		bytes = tonumber(bytes) or 0
	end

	return math.floor(bytes / map[unit])
end

function convert_to_bytes(value, unit)
	local map = {
		b = 1,
		kb = 1000,
		mb = 1000 * 1000,
		gb = 1000 * 1000 * 1000
	}

	if type(value) ~= "number" then
		value = tonumber(value) or 0
	end

	return value * map[unit]
end

function convert_bytes_auto(bytes)
	local units
	local map = {
		b = 1,
		kb = 1000,
		mb = 1000 * 1000,
		gb = 1000 * 1000 * 1000
	}

	if type(bytes) ~= "number" then
		bytes = tonumber(bytes) or 0
	end

	if bytes < map.kb then
		units = "b"
	elseif bytes < map.mb then
		units = "kb"
	elseif bytes < map.gb then
		units = "mb"
	else
		units = "gb"
	end

	return convert_bytes(bytes, units), units
end

function convert_seconds(sec, unit)
	local map = {
		sec = 1,
		min = 60,
		hr = 60 * 60,
		day = 60 * 60 * 24
	}

	if type(sec) ~= "number" then
		sec = tonumber(sec) or 0
	end

	return math.floor(sec / map[unit])
end

function convert_seconds_auto(sec)
	local units
	local map = {
		sec = 1,
		min = 60,
		hr = 60 * 60,
		d = 60 * 60 * 24
	}

	if type(sec) ~= "number" then
		sec = tonumber(sec) or 0
	end

	if sec < map.min then
		units = "sec"
	elseif sec < map.hr then
		units = "min"
	elseif sec < map.d then
		units = "hr"
	else
		units = "d"
	end

	return convert_seconds(sec, units), units
end

function fw_version()
	return fs.readfile("/etc/version") or "RUTX_T_00.00.00"
end

function week_days()
	return {
			{1, "Monday", "mon"},
			{2, "Tuesday", "tue"},
			{3, "Wednesday", "wed"},
			{4, "Thursday", "thu"},
			{5, "Friday", "fri"},
			{6, "Saturday", "sat"},
			{0, "Sunday", "sun"}
		}
end

function net_type(value, key)
	if value and key then
		for _, net in ipairs(net_map) do
			if net[key] then
				local v
				local buff = type(net[key]) == "string" and {net[key]} or net[key]

				for _, v in ipairs(buff) do
					if string.find(value, v) then
						return net.type
					end
				end
			end
		end
	end

	return false
end

function enable_vpn_firewall(vpn_type, zone, zone_opt, rule_opt)

	local exists = false
	local sid

	if zone and zone_opt then
		if not fw.get_zone(zone, zone) then
			uci:section("firewall", "zone", nil, zone_opt)
		end

		if #fw.get_zone(zone, zone):get_forwardings_by("src") == 0 then
			fw.get_zone(zone, zone):add_forwarding_to("lan")
		end

		if #fw.get_zone(zone, zone):get_forwardings_by("dest") == 0 then
			fw.get_zone(zone, zone):add_forwarding_from("lan")
		end
	end

	uci:foreach("firewall", "rule",
		function(s)
			if vpn_type and s.vpn_type and s.vpn_type == vpn_type and rule_opt and s.name == rule_opt.name then
				exists = true
				sid = s[".name"]
				uci:delete("firewall", sid, "enabled")
			end
	end)

	if vpn_type and rule_opt and not exists then
		uci:section("firewall", "rule", nil, rule_opt)
	end
end

function disable_vpn_firewall(vpn_type)
	local sid
	if vpn_type then
		uci:foreach("firewall", "rule", function(s)
			if s.vpn_type == vpn_type then
				sid = s[".name"]
				if sid then
					uci:set("firewall", sid, "enabled", "0")
				end
			end
		end)

		uci:foreach("firewall", "redirect", function(s)
			if s.vpn_type == vpn_type then
				sid = s[".name"]
				if sid then
					uci:set("firewall", sid, "enabled", "0")
				end
			end
		end)
	end
end

function memory_usage()
	local sysinfo = util.ubus("system", "info") or {}
	local memory_usage = {}
	local meminfo = sysinfo.memory or {
		total = 0, free = 0, buffered = 0, shared = 0
	}

	local ram_total = tonumber(string.format("%.2f", meminfo.total / 1024 / 1024))
	local ram_free = tonumber(string.format("%.2f", (((meminfo.free + meminfo.buffered + meminfo.shared) / 1024) +
					util.trim(sys.exec("free | grep Mem | awk '{print $7}'"))) / 1024))
	local ram_used = string.format("%.2f", ram_total - ram_free)
	local flash_used = util.trim(sys.exec("df -k | grep -w overlayfs | tail -1 | awk '{print $3}'")) or 0
	local flash_total = util.trim(sys.exec("df -k | grep -w overlayfs | tail -1 | awk '{print $2}'")) or 0

	memory_usage = {
		ram_total			= ram_total,
		ram_free			= ram_free,
		ram_used			= ram_used,
		ram_buffered		= tonumber(string.format("%.2f", meminfo.buffered / 1024 / 1024)),
		ram_shared			= tonumber(string.format("%.2f", meminfo.shared / 1024 / 1024)),
		ram_percentage		= string.format("%.2f", 100 * ram_used / ram_total),
		flash_used			= tonumber(string.format("%.2f", flash_used/1024)),
		flash_total			= tonumber(string.format("%.2f", flash_total/1024)),
		flash_percentage	= string.format("%.2f", 100 * flash_used / flash_total)
	}

	return memory_usage
end

function ls(path)
	local files = {}
	local list = io.popen("ls %s" % path)

	if list then
		while true do
			local file = list:read("*l")
			if not file then
				break
			else
				files[#files+1] = file
			end
		end

		list:close()

		return files
	end

	return files
end

function find(path, name)
	local files = {}
	local list = io.popen("find %s -name %s" % {path or "", name})

	if list then
		while true do
			local file = list:read("*l")
			if not file then
				break
			else
				files[#files+1] = file
			end
		end

		list:close()

		return files
	end

	return files
end
